'''
Usage: counted_example.py --help
       counted_example.py -v...
       counted_example.py go [go]
       counted_example.py (--path=<path>)...
       counted_example.py <file> <file>

Try: counted_example.py -vvvvvvvvvv
     counted_example.py go go
     counted_example.py --path ./here --path ./there
     counted_example.py this.txt that.txt
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__))



from docpie.element import Atom, Option, Command, Argument
from docpie.element import Optional, Required, OptionsShortcut
from docpie.element import Either
from docpie.tokens import Token
from docpie.error import DocpieError

import logging
import re
import textwrap
import warnings

logger = logging.getLogger('docpie.parser')


class Parser(object):

    option_name_2_instance = {}

    def _parse_pattern(self, token):
        logger.debug('get token %s', token)
        elements = []
        while token:
            atom = token.current()
            if atom in '([':
                elements.append(self._parse_bracket(token))
            elif atom == '|':
                elements.append(token.next())
            else:
                assert atom != '...', 'fix me: unexpected "..." when parsing'
                elements.extend(self._parse_element(token))

        logger.debug(elements)
        if '|' in elements:
            elements[:] = [self._parse_pipe(elements)]
        logger.debug('parsed result %s', elements)
        return elements

    def _parse_bracket(self, token):
        start = token.next()
        instance_type = Required if start == '(' else Optional

        lis = token.till_end_bracket(start)

        repeat = False
        while token.current() == '...':
            repeat = True
            token.next()

        bracket_token = Token(lis)

        instances = self._parse_pattern(bracket_token)

        return instance_type(*instances, **{'repeat': repeat})

    def _parse_element(self, token):
        atom = token.next()
        assert atom is not None
        if atom == '[options]':
            # `[options] ...`? Note `[options...]` won't work
            repeat = False
            while token and token.current() == '...':
                repeat = True
                token.next()
            if repeat:
                ins = Optional(OptionsShortcut(self.options), repeat=True)
            else:
                ins = OptionsShortcut(self.options)
            return (ins,)

        if atom in ('-', '--'):
            if token.current() == '...':
                repeat = True
                token.next()
            else:
                repeat = False
            return (Command(atom, repeat=repeat),)

        flag = None
        arg_token = None
        prepended = False
        opt_2_ins = self.option_name_2_instance

        # --all=<sth>... -> --all=<sth> ...
        # --all=(<sth> <else>)... -> --all= ( <sth> <else> ) ...
        if atom.startswith('--') and '=' in atom:
            flag_, arg_ = atom.split('=', 1)
            if Atom.get_class(flag_) is Option:
                flag = flag_
                if arg_:
                    arg_token = Token([arg_])
                else:
                    next_arg = token.next()
                    if next_arg not in ('(', '['):
                        raise DocpieError('format error: %s' % atom)
                    tk = [next_arg]
                    tk.extend(token.till_end_bracket(next_arg))
                    tk.append(')' if next_arg == '(' else ']')
                    arg_token = Token(tk)

                if token.current() == '...':
                    arg_token.append(token.next())

            # if '<' in atom and atom.endswith('>'):
            #  and not self.stdopt:
            #     index = atom.index('<')
            #     flag_, arg_ = atom[:index], atom[index:]
            #     if Atom.get_class(flag_) is Option:
            # and Atom.get_class(arg_) is Argument:
            #         flag = flag_
            #         arg_token = Token([arg_])

        # -a<sth> -aSTH -asth, -a, -abc<sth>
        elif atom.startswith('-') and not atom.startswith('--'):
            rest = None

            lt_index = atom.find('<')
            # -a<sth> -abc<sth>
            if lt_index >= 0 and atom.endswith('>'):
                # -a<sth> -> -a <sth>; -abc<sth> -> -a -bc<sth>
                if self.stdopt and self.attachopt:
                    if not self.attachvalue:
                        raise DocpieError(
                            "You can't write %s while attachvalue=False" %
                            atom)
                    flag_ = atom[:2]
                    # -abc<sth>
                    if lt_index > 2:
                        token.insert(0, '-' + atom[2:])
                        prepended = True
                    else:
                        arg_token = Token([atom[lt_index:]])
                # -a<sth> -> -a <sth>; -abc<sth> -> -abc <sth>
                else:
                    flag_ = atom[:lt_index]
                    arg_token = Token([atom[lt_index:]])
            else:
                flag_, rest = atom[:2], atom[2:]

            if Atom.get_class(flag_) is Option:
                flag = flag_
                # announced in Options
                if flag in opt_2_ins:
                    ins = opt_2_ins[flag_][0]
                    # In Options it requires no argument
                    if ins.ref is None:
                        # sth stacked with it
                        if rest:
                            if Atom.get_class(rest) is Argument:
                                raise DocpieError(
                                    ('%s announced difference in '
                                     'Options(%s) and Usage(%s)') %
                                    (flag_, ins, atom))
                            if not (self.stdopt and self.attachopt):
                                raise DocpieError(
                                    ("You can't write %s while it requires "
                                     "argument and attachopt=False") % atom)

                            token.insert(0, '-' + rest)
                            prepended = True
                    # In Options it requires argument
                    else:
                        if rest:
                            arg_token = Token([rest])
                        else:
                            _current = token.current()
                            if _current in '([':
                                tk = [token.next()]
                                tk.extend(token.till_end_bracket(tk[0]))
                                tk.append(')' if tk[0] == '(' else ']')
                                arg_token = Token(tk)
                            elif _current in ('...', '|'):
                                raise DocpieError(
                                    ('%s requires argument in Options(%s) '
                                     'but hit "%s" in Usage') %
                                    atom, ins, _current)
                            else:
                                arg_token = Token([token.next()])

                        if token.current() == '...':
                            arg_token.append(token.next())
                elif rest:
                    if not (self.stdopt and self.attachopt):
                        raise DocpieError(
                            "You can't write %s while it requires argument "
                            "and attachopt=False" % atom)
                    # -asth -> -a -sth
                    token.insert(0, '-' + rest)
                    prepended = True

        if flag is not None:
            logger.debug('parsing flag %s, %s, %s', flag, arg_token, token)
            if arg_token is not None:
                ref_lis = self._parse_pattern(Token(arg_token))
                ref = Required(*ref_lis).fix()
            else:
                ref = None
            ins = Option(flag, ref=ref)

            if flag in opt_2_ins:
                opt_ins = opt_2_ins[flag][0]
                ins.names.update(opt_ins.names)
                # != won't work on pypy
                if not (ins == opt_ins):
                    raise DocpieError(
                        '%s announces differently in '
                        'Options(%r) and Usage(%r)' %
                        (atom, opt_ins, ins))
            if token.current() == '...':
                ins = Required(ins, **{'repeat': True})
                token.next()

            return (ins,)

        if prepended:
            token.pop(0)

        atom_class = Atom.get_class(atom)
        ins = atom_class(atom)

        if token.current() == '...':
            token.next()
            # Not work on py2.6
            # ins = Required(*ins_lis, repeat=True)
            ins = Required(ins, **{'repeat': True})

        return (ins,)

    def _parse_pipe(self, lis):
        assert '...' not in lis
        assert len(lis) >= 3
        assert lis[0] != '|' != lis[-1]
        # assert lis[0] != '|'    # don't use lis[0] != lis[-1] != '|'
        # assert lis[-1] != '|'
        groups = [Required()]
        for each in lis:
            if each == '|':
                groups.append(Required())
            else:
                groups[-1].append(each)

        return Either(*groups)

    @staticmethod
    def drop_started_empty_lines(text):
        # drop the empty lines at start
        # different from lstrip
        logger.debug(repr(text))
        lis = text.splitlines()
        while lis and not lis[0].strip():
            lis.pop(0)

        return '\n'.join(lis)


class OptionParser(Parser):
    split_re = re.compile(r'(<.*?>)|\s+')
    wrap_symbol_re = re.compile(r'([\|\[\]\(\)]|\.\.\.)')
    line_re = re.compile(r'^(?P<indent> *)'
                         r'(?P<option>[\d\w=_, <>\-\[\]\.]+?)'
                         r'(?P<separater>$| $| {2,})'
                         r'(?P<description>.*?)'
                         r' *$',
                         flags=re.IGNORECASE)

    indent_re = re.compile(r'^(?P<indent> *)')
    to_space_re = re.compile(r',\s?|=')

    visible_empty_line_re = re.compile(r'^\s*?\n*|\n(:?[\ \t]*\n)+',
                                       flags=re.DOTALL)

    option_split_re_str = (r'([^\n]*{0}[\ \t]*\n?)')

    # split_re = re.compile(r'(<.*?>)|\s?')
    # default ::= chars "[default: " chars "]"
    # support xxxxxx.[default: ]
    # support xxxxxx.[default: yes]
    # not support xxxxx[default: no].
    # not support xxxxx[default: no]!
    # If you want to match a not so strict format, this may help:
    # default_re = re.compile(r'\[default: *(?P<default>.*?) *\]'
    #                         r' *'
    #                         r'[\.\?\!]? *$',
    #                         flags=re.IGNORECASE)
    default_re = re.compile(r'\[default: (?P<default>.*?)\] *$',
                            flags=re.IGNORECASE)

    def __init__(self, text, stdopt, attachopt, attachvalue):

        self.stdopt = stdopt
        self.attachopt = attachopt
        self.attachvalue = attachvalue

        self.name_2_instance = {}
        if text is None or not text.strip():    # empty
            self._opt_and_default_str = []
        else:
            self._opt_and_default_str = list(self._parse_text(text))

        self._chain = self._parse_to_instance(self._opt_and_default_str)

    def get_chain(self):
        return self._chain
        # return [Optional(x) for x in self._chain]

    @classmethod
    def parse(cls, text, name, case_sensitive=False):
        # TODO: allow options separate with single break line
        # currently:
        # different option section must be separated with at
        # least one visible empty line
        #
        # Options:
        #   -f, --flag     a flag
        #
        # Help Options:
        #   -o, --output   output stream

        es_name = re.escape(name)

        option_split_re = re.compile(
            cls.option_split_re_str.format(es_name),
            flags=re.DOTALL | (0 if case_sensitive else re.IGNORECASE)
        )

        raw_content = {}
        formal_collect = []

        try:
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                split = cls.visible_empty_line_re.split(text)
        except ValueError:  # python >= 3.5
            return None, {}

        for text in filter(lambda x: x and x.strip(), split):

            # logger.debug('get options group:\n%r', text)
            try:
                with warnings.catch_warnings():
                    warnings.simplefilter("ignore")
                    split_options = option_split_re.split(text)
            except ValueError:  # python >= 3.5
                continue

            split_options.pop(0)

            for title, section in zip(split_options[::2], split_options[1::2]):
                prefix, _, end = title.partition(name)

                prefix = prefix.strip()

                if prefix in raw_content:
                    raise DocpieError('duplicated options section %s' % prefix)

                section = section.rstrip()
                if end.endswith('\n'):
                    formal = section
                else:
                    formal = ' ' * len(title) + section

                formal_collect.append(formal)
                raw_content[prefix] = title + section

        if formal_collect:
            formal_result = '\n'.join(
                textwrap.dedent(x) for x in formal_collect)
        else:
            formal_result = None

        return formal_result, raw_content

    def _parse_text(self, text):
        collect = []
        to_list = text.splitlines()

        # parse first line. Should NEVER failed.
        # this will ensure in `[default: xxx]`,
        # the `xxx`(e.g: `\t`, `,`) will not be changed by _format_line
        previous_line = to_list.pop(0)
        collect.append(self._parse_line(previous_line))

        for line in to_list:
            indent_match = self.indent_re.match(line)
            this_indent = len(indent_match.groupdict()['indent'])

            if this_indent >= collect[-1]['indent']:
                # A multi line description
                previous_line = line
                continue

            # new option line
            # deal the default for previous option
            collect[-1]['default'] = self._parse_default(previous_line)
            # deal this option
            collect.append(self._parse_line(line))
            logger.debug(collect[-1])
            previous_line = line
        else:
            collect[-1]['default'] = self._parse_default(previous_line)

        return ((each['option'], each['default']) for each in collect)

    spaces_re = re.compile(r'(\ \ \s*|\t\s*)')

    @classmethod
    def _cut_first_spaces_outside_bracket(cls, string):
        right = cls.spaces_re.split(string)
        left = []
        if right and right[0] == '':    # re matches the start of the string
            right.pop(0)
        if right and not right[0].strip():    # it is indent
            left.append(right.pop(0))
        brackets = {'(': 0, '[': 0, '<': 0}
        close_brancket = {'(': ')', '[': ']', '<': '>'}
        cutted = ''

        while right:
            this = right.pop(0)
            for open_b in brackets:
                brackets[open_b] += this.count(open_b)
                brackets[open_b] -= this.count(close_brancket[open_b])
            if sum(brackets.values()):
                left.append(this)
            elif (not this.strip() and
                    len(this.expandtabs()) >= 2):
                cutted = this
                break
            else:
                left.append(this)
        return ''.join(left), cutted, ''.join(right)

    @classmethod
    def _parse_line(cls, line):
        opt_str, separater, description_str = \
                cls._cut_first_spaces_outside_bracket(line)

        logger.debug('%(line)s -> %(opt_str)r, '
                     '%(separater)r, '
                     '%(description_str)r' % locals())
        if description_str.strip():
            indent = len(opt_str.expandtabs()) + len(separater.expandtabs())
        else:
            indent = 2 + len(cls.indent_re.match(
                                 opt_str.expandtabs()
                            ).groupdict()['indent'])
        return {'option': opt_str.strip(), 'indent': indent}

    @classmethod
    def _parse_default(cls, line):
        m = cls.default_re.search(line)
        if m is None:
            return None
        return m.groupdict()['default']

    def _parse_to_instance(self, lis):
        opts = []
        for opt_str, default in lis:
            logger.debug('%s:%r' % (opt_str, default))
            opt, repeat = self._parse_opt_str(opt_str)
            opt.default = default
            result = Optional(opt, repeat=repeat)
            for name in opt.names:
                self.name_2_instance[name] = result
            opts.append(result)
        return opts

    def _split_short_by_cfg(self, s):
        if self.stdopt:
            if (not s.startswith('--') and
                    len(s) > 1):
                return s[:2], s[2:]
        return s, ''

    def _parse_opt_str(self, opt):

        repeat = False

        # -sth=<goes> ON -> -sth, <goes>, ON
        opt_lis = self._opt_str_to_list(opt)
        logger.debug('%r -> %s' % (opt, opt_lis))

        first = opt_lis.pop(0)
        if not first.startswith('-'):
            raise DocpieError('option %s does not start with "-"' % first)

        # if self.stdopt:
        # -sth -> name=-s, value=th
        # else:
        # -sth -> name=-sth, value=''
        name, value = self._split_short_by_cfg(first)
        opt_ins = Option(name)
        if value == '...':
            repeat = True
            # -f... <sth>
            if opt_lis and not opt_lis[0].startswith('-'):
                raise DocpieError(
                    'option "%s" has argument following "..."', opt)
        elif value:
            args_ins = [Required(Argument(value))]
        else:
            args_ins = []

        if opt_lis and opt_lis[0] == '...':
            repeat = True
            opt_lis.pop(0)
            if opt_lis and not opt_lis[0].startswith('-'):
                raise DocpieError(
                    'option "%s" has argument following "..."', opt)

        args = []    # store the current args after option
        for each in opt_lis:
            if each.startswith('-'):    # alias
                name, value = self._split_short_by_cfg(each)
                opt_ins.names.add(name)
                if value:
                    args_ins.append(Required(Argument(value)))
                if args:    # trun it into instance
                    if args[0] == '...':
                        if len(args) != 1:
                            raise DocpieError(
                                'Error in %s: "..." followed by non option',
                                opt)
                        repeat = True
                    else:
                        this_arg = Required(
                                            *self._parse_pattern(Token(args))
                                           ).fix()
                        if this_arg is not None:
                            args_ins.append(this_arg)
                del args[:]
            else:
                args.append(each)
        else:
            if args:    # trun it into instance
                if args[0] == '...':
                    if len(args) != 1:
                        raise DocpieError(
                            'Error in %s: "..." followed by non option',
                            opt)
                    repeat = True
                else:
                    this_arg = Required(
                        *self._parse_pattern(Token(args))).fix()
                    if this_arg is not None:
                        args_ins.append(this_arg)

        # option without any args
        if not args_ins:
            return opt_ins, repeat

        # in Option, there should only have one arg list
        # e.g.: -f <file> --file=FILE -> -f/--file (<file>|FILE)
        # because the arg name will now be shown, it parsed as:
        # -f <file> --file=FILE -> -f/--file (<file>)
        current_ins = args_ins.pop(0)
        current_range = current_ins.arg_range()

        # avoid e.g.: -f <a> <b> --file <c>
        for other_ins in args_ins:
            this_range = other_ins.arg_range()
            if this_range != current_range:
                raise DocpieError("%s announced differently (%s, %s)" % (
                                  opt_ins, this_range, current_range))

        if len(current_range) > 1:
            logger.info('too many possibilities: '
                        'option %s expect %s arguments',
                        name, '/'.join(map(str, current_range)))

        # TODO: check if current_ins contain Command(not allowed in fact)
        opt_ins.ref = current_ins
        return opt_ins, repeat

    def _opt_str_to_list(self, opt):
        dropped_comma_and_equal = opt.replace(',', ' ').replace('=', ' ')
        wrapped_space = self.wrap_symbol_re.sub(
            r' \1 ', dropped_comma_and_equal)
        opt_lis = [x for x in self.split_re.split(wrapped_space) if x]
        return opt_lis


class UsageParser(Parser):

    angle_bracket_re = re.compile(r'(<.*?>)')
    wrap_symbol_re = re.compile(r'(\[options\]|\.\.\.|\||\[|\]|\(|\))')
    split_re = re.compile(r'(\[options\]|\S*<.*?>\S*)|\s+')
    # will match '-', '--', and
    # flag ::= "-" [ "-" ] chars "=<" chars ">"
    # it will also match '---flag', so use startswith('---') to check
    std_flag_eq_arg_re = re.compile(r'(?P<flag>^-{1,2}[\da-zA-Z_\-]*)'
                                    r'='
                                    r'(?P<arg><.*?>)'
                                    r'$')

    usage_re_str = (r'(?:^|\n)'
                    r'(?P<raw>'
                     r'(?P<name>[\ \t]*{0}[\ \t]*)'
                     r'(?P<sep>\n?)'
                     r'(?P<section>.*?)'
                    r')'
                    r'\s*'
                    r'(?:\n\s*\n|\n\s*$|$)')

    def __init__(self, text, name, options, option_name_2_instance,
                 stdopt, attachopt, attachvalue):

        self.option_name_2_instance = option_name_2_instance
        self.options = options

        self.stdopt = stdopt
        self.attachopt = attachopt
        self.attachvalue = attachvalue
        self._chain = self._parse_text(text, name)

    @classmethod
    def parse(cls, text, name, case_sensitive=False):
        name = re.escape(name)
        match = re.search(
            cls.usage_re_str.format(name),
            text,
            flags=re.DOTALL | (0 if case_sensitive else re.IGNORECASE))

        if match is None:
            return None, None
        dic = match.groupdict()
        logger.debug(dic)
        if dic['sep'] == '\n':
            return dic['section'], dic['raw']
        reallen = len(dic['name'])
        replace = ''.ljust(reallen)
        drop_name = match.expand('%s\g<sep>\g<section>' % replace)
        return cls.drop_started_empty_lines(drop_name).rstrip(), dic['raw']

    def _parse_text(self, text, name):
        result = []
        for each_line in self._split_line_by_indent(text):
            raw_str_lis = self._parse_line_to_lis(each_line, name)
            chain = self._parse_pattern(Token(raw_str_lis))
            result.append(chain)
        return result

    indent_re = re.compile(r'^ *')

    def _split_line_by_indent(self, text):
        lines = text.splitlines()
        if len(lines) == 1:
            yield lines[0]
            return

        first_line = lines.pop(0)
        line_to_join = [first_line]
        indent = len(
            self.indent_re.match(first_line.expandtabs()).group())
        while lines:
            this_line = lines.pop(0)
            this_indent = len(
                self.indent_re.match(this_line.expandtabs()).group())

            if this_indent > indent:
                line_to_join.append(this_line)
            else:
                yield ''.join(line_to_join)
                line_to_join[:] = (this_line,)
                indent = len(
                    self.indent_re.match(this_line.expandtabs()).group())

        else:
            yield ' '.join(line_to_join)

    def _parse_line_to_lis(self, line, name=None):
        if name is not None:
            _, find_name, line = line.partition(name)
            if not find_name:
                raise DocpieError(
                    '%s is not in usage pattern %s' % (name, _))

        # wrapped_space = self.wrap_symbol_re.sub(r' \1 ', line.strip())
        # logger.debug(wrapped_space)
        # result = [x for x in self.split_re.split(wrapped_space) if x]

        angle_bracket_re = self.angle_bracket_re
        wrap_symbol_re = self.wrap_symbol_re

        try:
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                sep_by_angle = angle_bracket_re.split(line)
        except ValueError:
            sep_by_angle = [line]

        wrap_space = []
        for index, each_block in enumerate(sep_by_angle):
            if index % 2:
                wrap_space.append(each_block)
                continue

            if not each_block:
                continue

            warped_space = wrap_symbol_re.sub(r' \1 ', each_block)

            wrap_space.append(warped_space)

        wraped = ''.join(wrap_space)

        try:
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                sep = self.split_re.split(wraped)
        except ValueError:
            sep = [wraped]

        result = list(filter(None, sep))

        # drop name
        if name is None:
            result.pop(0)

        return result

    @classmethod
    def _find_optionshortcut_and_outside_option_names(cls, lis):
        opt_ouside = []
        opt_cuts = []
        for element in lis:
            if isinstance(element, OptionsShortcut):
                opt_cuts.append(element)
            elif isinstance(element, list):
                outside, srtcuts = \
                    cls._find_optionshortcut_and_outside_option_names(element)
                opt_ouside.extend(outside)
                opt_cuts.extend(srtcuts)
            elif isinstance(element, Option):
                opt_ouside.append(element)

        return opt_ouside, opt_cuts

    def get_chain_and_all_options(self):
        result = []
        all_options = [x[0] for x in self.option_name_2_instance.values()]
        for each_usage in self._chain:
            ins = Required(*each_usage).fix()
            if ins is None:
                result.append(Optional())
                continue

            outside_opts, opt_shortcuts = \
                self._find_optionshortcut_and_outside_option_names(ins)

            for opt_cut in opt_shortcuts:
                for opt_ins in outside_opts:
                    opt_cut.set_hide(opt_ins.names)

            all_options.extend(outside_opts)

            for usage in ins.expand_either():
                usage.push_option_ahead()
                result.append(usage)

        return result, all_options



'''
usage:
  python git.py add [options] [--] [<filepattern>...]

Options:
    -h, --help
    -n, --dry-run        dry run
    -v, --verbose        be verbose
    -i, --interactive    interactive picking
    -p, --patch          select hunks interactively
    -e, --edit           edit current diff and apply
    -f, --force          allow adding otherwise ignored files
    -u, --update         update tracked files
    -N, --intent-to-add  record only the fact that the path will be added later
    -A, --all            add all, noticing removal of tracked files
    --refresh            don't add, only refresh the index
    --ignore-errors      just skip files which cannot be added because of errors
    --ignore-missing     check if - even missing - files are ignored in dry run
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__, name='git.py'))



"""
Example for docpie, linux cp-like command


NAME
     cp -- copy files

USAGE:
     cp [options] <source_file> ... <target_directory>

OPTIONS:
     -f    If the destination file cannot be opened, remove it and create a
           new file, without prompting for confirmation regardless of its per-
           missions.
     -v, --verbose
           Cause cp to be verbose, showing files as they are copied.
     -X    Do not copy Extended Attributes (EAs) or resource forks.
     -R    If source_file designates a directory, cp copies the directory and
           the entire subtree connected at that point.
"""

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__))



'''
usage: python git.py checkout [options] <branch>
       python git.py checkout [options] <branch> -- <file>...

options:
    -q, --quiet           suppress progress reporting
    -b <branch>           create and checkout a new branch
    -B <branch>           create/reset and checkout a branch
    -l                    create reflog for new branch
    -t, --track           set upstream info for new branch
    --orphan <new branch>
                          new unparented branch
    -2, --ours            checkout our version for unmerged files
    -3, --theirs          checkout their version for unmerged files
    -f, --force           force checkout (throw away local modifications)
    -m, --merge           perform a 3-way merge with the new branch
    --conflict <style>    conflict style (merge or diff3)
    -p, --patch           select hunks interactively
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__, name='git.py'))



'''
usage: python git.py branch [options]
                            [-r | -a] [--merged=<commit> | --no-merged=<commit>]
       python git.py branch [options] [-l] [-f] <branchname> [<start-point>]
       python git.py branch [options] [-r] (-d | -D) <branchname>
       python git.py branch [options] (-m | -M) [<oldbranch>] <newbranch>

options:
    -h, --help
    -v, --verbose         show hash and subject, give twice for upstream branch
    -t, --track           set up tracking mode (see git-pull(1))
    --set-upstream        change upstream info
    --color=<when>        use colored output
    -r                    act on remote-tracking branches
    --contains=<commit>   print only branches that contain the commit
    --abbrev=<n>          use <n> digits to display SHA-1s
    -a                    list both remote-tracking and local branches
    -d                    delete fully merged branch
    -D                    delete branch (even if not merged)
    -m                    move/rename a branch and its reflog
    -M                    move/rename a branch, even if target exists
    -l                    create the branch's reflog
    -f, --force           force creation (when already exists)
    --no-merged=<commit>  print only not merged branches
    --merged=<commit>     print only merged branches
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__, name='git.py'))



import logging
from docpie.error import DocpieError

logger = logging.getLogger('docpie.tokens')


class Token(list):
    _brackets = {'(': ')', '[': ']'}  # , '{': '}', '<': '>'}

    def next(self):
        return self.pop(0) if self else None

    def current(self):
        return self[0] if self else None

    def till_end_bracket(self, start):
        # start bracket should not in self
        end = self._brackets[start]
        count = dict.fromkeys(self._brackets, 0)
        count[start] = 1
        element = []
        while self:
            this = self.pop(0)
            for each_start, each_end in self._brackets.items():
                count[each_start] += (this == each_start)
                count[each_start] -= (this == each_end)
            if this == end and all(x == 0 for x in count.values()):
                if any(x < 0 for x in count.values()):
                    raise DocpieError("brackets not in pair")
                return element
            element.append(this)
        else:
            raise DocpieError("brackets not in pair")


class Argv(list):

    def __init__(self, argv, auto2dashes,
                 stdopt, attachopt, attachvalue, known=[]):

        super(Argv, self).__init__(argv)
        self.auto_dashes = auto2dashes
        self.dashes = False
        # when this is on, only --option can try to match.
        self.option_only = False
        self.stdopt = stdopt
        self.attachopt = attachopt
        self.attachvalue = attachvalue
        self.error = None
        self.known = known

    def formal(self, options_first):
        names = self.known
        result = []
        for index, each in enumerate(self):
            # first command/argument
            if (options_first and
                    (each in ('-', '--') or not each.startswith('--'))):
                result.extend((each, '--'))  # add '--' after it
                result.extend(self[index + 1:])
                break

            if each == '--' and self.auto_dashes:
                result.extend(self[index:])
                break

            if each == '-':
                result.append(each)
                continue

            # this won't work when -abc, and it actually means -a -b -c,
            # but -b, -c is not defined
            if each.startswith('-') and not each.startswith('--'):
                if self.stdopt:
                    this_opt = each[:2]
                else:
                    this_opt = each

                if this_opt not in names:
                    self.error = 'Unknown option: %s.' % this_opt
                    result.extend(self[index:])
                    break

                result.append(each)

            elif not each.startswith('--') or each in names:
                result.append(each)
            else:
                option, equal, value = each.partition('=')
                if option == '--' or option in names:
                    result.append(each)
                else:
                    possible = list(
                        filter(lambda x: x.startswith(option), names))
                    if not possible:
                        self.error = 'Unknown option: %s.' % option
                        result.extend(self[index:])
                        break
                        # Don't raise. It may be --help
                        # and the developer didn't announce in
                        # either Usage or Options
                        # logger.info('not found this argv %s', each)
                        # result.append(each)
                    elif len(possible) == 1:
                        replace = ''.join((possible[0], equal, value))
                        logger.debug('expand %s -> %s', each, replace)
                        result.append(replace)
                    else:
                        self.error = (
                            '%s is not a unique prefix: %s?' %
                            (option, ', '.join(possible))
                        )
                        result.extend(self[index:])
                        break

        logger.debug('%s -> %s', self, result)
        self[:] = result
        return None

    def current(self, offset=0):
        return self[offset] if len(self) > offset else None

    def insert(self, index, object):
        if self.auto_dashes and '--' in self:
            dashes_index = self.index('--')
        else:
            dashes_index = float('inf')

        fine = True
        if (object != '-' and
                object.startswith('-') and
                not object.startswith('--')):
            if self.stdopt:
                flag = object[:2]
            else:
                flag = object
            fine = (flag in self.known)
            logger.debug('%s has flag %s, is known: %s', object, flag, fine)

        if (index <= dashes_index and fine or
                index > dashes_index or
                fine):
            logger.debug('insert %s into %s at %s', object, self, index)
            return super(Argv, self).insert(index, object)

        logger.info('%s not in %s', flag, self.known)
        self.error = 'Unknown option: %s.' % flag

    def break_for_option(self, names):
        auto_dashes = self.auto_dashes
        stdopt = self.stdopt
        attachvalue = self.attachvalue
        for index, option in enumerate(self):
            if option == '--' and auto_dashes:
                return False, None, 0
            for name in names:
                if option.startswith(name):
                    value = None
                    self.pop(index)
                    # `-flag=sth` -> `-flag =sth`
                    if not stdopt:
                        _, _, value = option.partition(name)
                        if not value:
                            value = None

                    # `-flag` -> `-f lag`
                    # `--flag=sth` -> `--flag sth`
                    # `--flag=` -> `--flag ""`
                    # `-f` -> `-f None`
                    elif attachvalue:
                        _, _, value = option.partition(name)
                        if name.startswith('--') and value.startswith('='):
                            value = value[1:]
                        elif not value:
                            value = None

                    logger.debug('find %s, attached %s, index %s of %s',
                                 name, value, index, self)
                    return True, value, index

        return False, None, 0

    def next(self, offset=0):
        return self.pop(offset) if len(self) > offset else None

    def check_dash(self):
        if not self:
            return
        if self[0] == '--':
            self.dashes = True

    def clone(self):
        result = Argv(self, self.auto_dashes,
                      self.stdopt, self.attachopt, self.attachvalue)
        result.dashes = self.dashes
        result.option_only = self.option_only
        result.error = self.error
        result.known = self.known
        return result

    def restore(self, ins):
        self[:] = ins
        self.dashes = ins.dashes
        self.option_only = ins.option_only
        self.error = ins.error

    def status(self):
        # return len(self)
        return list(self)

    def dump_value(self):
        return list(self), self.dashes, self.option_only

    def load_value(self, value):
        self[:], self.dashes, self.option_only = value



'''
usage: python git.py remote [-v | --verbose]
       python git.py remote add [-t <branch>] [-m <master>] [-f] [--mirror] <name> <url>
       python git.py remote rename <old> <new>
       python git.py remote rm <name>
       python git.py remote set-head <name> (-a | -d | <branch>)
       python git.py remote [-v | --verbose] show [-n] <name>
       python git.py remote prune [-n | --dry-run] <name>
       python git.py remote [-v | --verbose] update [-p | --prune] [(<group> | <remote>)...]
       python git.py remote set-branches <name> [--add] <branch>...
       python git.py remote set-url <name> <newurl> [<oldurl>]
       python git.py remote set-url --add <name> <newurl>
       python git.py remote set-url --delete <name> <url>

options:
    -v, --verbose         be verbose; must be placed before a subcommand
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__, name='git.py'))



"""
NAME
       dsa - DSA key processing

USAGE:
       dsa [-inform (PEM|DER)] [-outform (PEM|DER)] [-in <filename>]
           [-passin <arg>] [-out <filename>] [-passout <arg>] [-aes128] [-aes192]
           [-aes256] [-camellia128] [-camellia192] [-camellia256] [-des] [-des3]
           [-idea] [-text] [-noout] [-modulus] [-pubin] [-pubout] [-engine <id>]

OPTIONS:
       -inform DER|PEM
           This specifies the input format.
       -outform DER|PEM
           This specifies the output format
       -in <filename>
           This specifies the input filename to read a key from or standard
           input if this option is not specified.
       -passin <arg>
           the input file password source.
       -out <filename>
           This specifies the output filename to write a key to or standard
           output by is not specified.
       -passout <arg>
           the output file password source.
       -text
           prints out the public, private key components and parameters.
       -noout
           this option prevents output of the encoded version of the key.
       -modulus
           this option prints out the value of the public key component of the
           key.
       -pubin
           by default a private key is read from the input file: with this
           option a public key is read instead.
       -pubout
           by default a private key is output.
       -engine <id>
           specifying an engine (by its unique id string) will cause dsa to
           attempt to obtain a functional reference to the specified engine,
           thus initialising it if needed.


EXAMPLES
       To remove the pass phrase on a DSA private key:

           dsa -in key.pem -out keyout.pem

       To encrypt a private key using triple DES:

           dsa -in key.pem -des3 -out keyout.pem

       To convert a private key from PEM to DER format:

           dsa -in key.pem -outform DER -out keyout.der

       To print out the components of a private key to standard output:

           dsa -in key.pem -text -noout

       To just output the public part of a private key:

           dsa -in key.pem -pubout -out pubkey.pem
"""

from docpie import docpie, bashlog

if __name__ == '__main__':
    print(docpie(__doc__))



import unittest
import logging
from docpie.parser import OptionParser, UsageParser
logger = logging.getLogger('docpie.test.parse_section')


class OptionsTest(unittest.TestCase):

    def p(self, text, name='Options:', case_sensitive=False):
        return OptionParser.parse(text, name, case_sensitive)

    def test_parse(self):
        doc = '''
        Not Part

            Options:
                -a
                -b
                -c

            Global Options:
                -d
                -e
                -f

            Command Options:
                -g
                -h


            Inline Options: -i
                            -j



            Inline2 Options:   -k
                               -l
            Nearer Options: -m
                             -n

        Not Part'''
        fmted, dic = self.p(doc)
        # print(fmted)
        # for k, v in dic.items():
        #     print('%10r: %r' % (k, v))
        self.assertEqual(
            fmted,
            '-a\n-b\n-c\n-d\n-e\n-f\n-g\n-h\n-i\n-j\n-k\n-l\n-m\n -n')
        self.assertEqual(
            dic,
            {'': ('            Options:\n'
                  '                -a\n'
                  '                -b\n'
                  '                -c'),
             'Global': ('            Global Options:\n'
                        '                -d\n'
                        '                -e\n'
                        '                -f'),
             'Inline2': ('            Inline2 Options:   -k\n'
                         '                               -l'),
             'Command': ('            Command Options:\n'
                         '                -g\n'
                         '                -h'),
             'Nearer': ('            Nearer Options: -m\n'
                        '                             -n'),
             'Inline': ('            Inline Options: -i\n'
                        '                            -j')})

class UsageTest(unittest.TestCase):

    def p(self, text, name='Usage:', case_sensitive=False):
        return UsageParser.parse(text, name, case_sensitive)

    def test_parse(self):
        doc = '''
        Usage: line1
               line2
               line3
               line4

           not'''

        expect = '''\
               line1
               line2
               line3
               line4'''

        expect2 = '''\
        Usage: line1
               line2
               line3
               line4'''

        result = self.p(doc)
        self.assertEqual(result[0], expect)
        self.assertEqual(result[1], expect2)

if __name__ == '__main__':
    # from docpie.bashlog import stdoutlogger, DEBUG
    # stdoutlogger(None, DEBUG)
    unittest.main()
    # OptionsTest().test_parse()
    # UsageTest().test_parse()


'''
usage: python git.py push [options] [<repository> [<refspec>...]]

options:
    -h, --help
    -v, --verbose         be more verbose
    -q, --quiet           be more quiet
    --repo <repository>   repository
    --all                 push all refs
    --mirror              mirror all refs
    --delete              delete refs
    --tags                push tags (can't be used with --all or --mirror)
    -n, --dry-run         dry run
    --porcelain           machine-readable output
    -f, --force           force updates
    --thin                use thin pack
    --receive-pack <receive-pack>
                          receive pack program
    --exec <receive-pack>
                          receive pack program
    -u, --set-upstream    set upstream for git pull/status
    --progress            force progress reporting
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__, name='git.py'))



import logging
import re
try:
    from itertools import product
except ImportError:
    # python 2.6
    def product(*args, **kwds):
        # product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
        # product(range(2), repeat=3) --> 000 001 010 011 100 101 110 111
        pools = map(tuple, args) * kwds.get('repeat', 1)
        result = [[]]
        for pool in pools:
            result = [x+[y] for x in result for y in pool]
        for prod in result:
            yield tuple(prod)

__all__ = ('Atom', 'Command', 'Argument', 'Option',
           'Unit', 'Required', 'Optional', 'OptionsShortcut', 'Either',
           'convert_2_dict', 'convert_2_object')

from docpie.tokens import Argv

logger = logging.getLogger('docpie.element')

try:
    StrType = basestring
except NameError:
    StrType = str
NoneType = type(None)


class Atom(object):
    # when stdopt=True, long option should only start with `--`
    # Otherwith, `-flag` is also long options
    # and `-flag=sth` will be considered as `-flag` and `=sth`
    # and attachopt won't work
    # and attachvalue is forced to be `True`

    # short option should not use `=`,
    # `-asth` equals `-a sth`, `-a=sth` equals `-a =sth`
    # long option can use
    flag_or_upper_re = re.compile(r'^(?P<hyphen>-{0,2})'
                                  r'($|[\da-zA-Z_][\da-zA-Z_\-]*$)')
    angular_bracket_re = re.compile(r'^<.*?>$')

    def __init__(self, *names, **kwargs):
        self.names = set(names)
        self.default = kwargs.get('default', None)
        self.value = None

    def _cache(get_class):
        cache_dic = {}

        def wrapped(cls, atom):
            if atom not in cache_dic:
                result = get_class(cls, atom)
                cache_dic[atom] = result
            return cache_dic[atom]

        return wrapped

    @classmethod
    @_cache
    def get_class(cls, atom):
        if atom in ('-', '--'):
            return Command
        elif atom == '-?':
            return Option

        m = cls.flag_or_upper_re.match(atom)
        if m:
            match = m.groupdict()
            if match['hyphen']:
                return Option
            elif atom.isupper():
                return Argument
            else:
                return Command
        elif cls.angular_bracket_re.match(atom):
            return Argument
        else:
            logger.debug('I guess %s is a Command' % atom)
            return Command

    def arg_range(self):
        return [1]

    def fix(self):
        return self
    fix_nest = fix
    fix_empty = fix

    @property
    def repeat(self):
        return False

    def dump_value(self):
        return self.value

    def load_value(self, value):
        self.value = value

    def expand_either(self):
        return [self]

    @classmethod
    def convert_2_dict(cls, obj):
        return {
            '__class__': obj.__class__.__name__,
            'names': tuple(obj.names),
            'default': obj.default,
        }

    @classmethod
    def convert_2_object(cls, dic):
        names = dic['names']
        cls_name = dic['__class__']
        assert cls_name in ('Option', 'Command', 'Argument')
        if cls_name == 'Option':
            # raise NotImplementedError('call Option.convert_2_object instead')
            return Option.convert_2_object(dic)
        elif cls_name == 'Command':
            cls = Command
        elif cls_name == 'Argument':
            cls = Argument
        default = dic['default']
        # Not work on py2.6
        # return cls(*names, default=default)
        return cls(*names, **{'default': default})

    def __str__(self):
        return '/'.join(self.names)

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, ', '.join(self.names))

    def __eq__(self, other):
        if type(self) is not type(other):
            return False
        if not self.names.intersection(other.names):
            return False
        return True


class Option(Atom):
    long_re = re.compile(r'(?P<name>--[0-9a-zA-Z]*)(?P<eq>=?)(?P<value>.*)')

    def __init__(self, *names, **kwargs):
        # assert all(x.startswith('-') for x in names)
        super(Option, self).__init__(*names, **kwargs)
        self.ref = kwargs.get('ref', None)    # a instance, not a list
        # self.countable = False

    def get_value(self, appeared_only, in_repeat):
        # default = self.default
        names = self.names
        value = self.value
        ref = self.ref

        multi = (in_repeat or (value is not True and
                               value is not False and
                               not isinstance(value, StrType)))

        if ref is not None:
            max_value_num = -1 if in_repeat else max(ref.arg_range())
            multi = multi or (max_value_num > 1)
        else:
            max_value_num = 0

        if not value:  # False/None/0
            if max_value_num == 0:
                if value is None:
                    value = False
            elif max_value_num == 1:
                value = None
            else:
                value = []

            result = {}
            if appeared_only:
                value = -1  # means never appeared

            for name in names:
                result[name] = value
            logger.debug('%s returns its value %s', self, result)
            return result

        # has value, has ref
        if ref is not None:
            if max_value_num != 0:
                value = ref.get_flat_list_value()
                if max_value_num == 1:
                    assert len(value) < 2, value
                    value = value[0] if value else None

        # has value, no ref
        elif multi:
            if value is None:
                value = 0
            value = int(value)

        logger.debug('%s returns its value %s', self, value)
        # Not work on py2.6
        # return {name: value for name in names}
        result = {}
        for name in names:
            result[name] = value
        return result

    def get_sys_default_value(self, appeared_only, in_repeat):
        if appeared_only and self.value is None:
            return {}

        if in_repeat:
            if self.ref is not None:
                value = []
            else:
                value = 0
        elif self.ref is not None:
            arg_range = self.ref.arg_range()
            max_arg_num = max(arg_range)
            if max_arg_num == 0:
                value = False
            elif max_arg_num == 1:
                value = None
            else:
                value = []
        else:
            value = False
        # Not work on py2.6
        # return {name: value for name in self.names}
        result = {}
        for name in self.names:
            result[name] = value
        return result

    def match(self, argv, repeat_match):
        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False

        if not repeat_match and self.value:
            logger.debug('%s already has a value', self)
            return True

        current = argv.current()
        if current is None:
            logger.info('no argv left')
            return False

        self_value = self.dump_value()
        argv_value = argv.dump_value()
        # saver.save(self, argv)

        find_it, attached_value, index = \
            argv.break_for_option(self.names)

        if not find_it:
            logger.debug('not found matching %s in %s', self, argv)
            argv.load_value(argv_value)
            # saver.rollback(self, argv)
            return False

        if repeat_match:
            if self.value is None:
                self.value = 1
            else:
                self.value += 1
        else:
            self.value = True

        if self.ref is None:
            if attached_value is not None:
                if argv.stdopt and argv.attachopt:
                    logger.debug("%s put -%s pack to argv %s",
                                 self, attached_value, argv)
                    argv.insert(index, '-' + attached_value)
                    logger.debug('%s matched %s / %s', self, self.value, argv)
                    return True
                logger.debug("%s doesn't have ref but argv is %s",
                             self, attached_value)
                # need restore?
                # self.load_value(self_value)
                # argv.load_value(argv_value)
                return False
            logger.debug('%s matched %s / %s', self, self.value, argv)
            return True

        # option expects a value(e.g. `--flag=<sth>`), but the argv looks like
        # --flag -- sth
        # then it should fail
        if (attached_value is None and
                argv.current(index) == '--' and
                argv.auto_dashes and
                min(self.ref.arg_range()) > 0):
            logger.info(
                '%s ref must fully match but failed because `--`', self)
            argv.error = ('/'.join(self.names) +
                          ' requires argument(s) but hits "--".')

            return False

        if attached_value is None:
            to_match_ref_argv = argv.clone()
            to_match_ref_argv[:] = argv[index:]
            del argv[index:]
        else:
            to_match_ref_argv = argv.clone()
            to_match_ref_argv[:] = [attached_value]

        result = self.ref.match(to_match_ref_argv, repeat_match)

        if attached_value is not None and to_match_ref_argv:
            logger.info('%s ref must fully match but failed for %s',
                        self, to_match_ref_argv)
            argv.error = ('%s requires argument(s).' % ('/'.join(self.names)))
            return False

        if not result:
            logger.debug('%s ref match failed %s', self, to_match_ref_argv)
            # saver.rollback(self, argv)
            self.load_value(self_value)
            argv.load_value(argv_value)
            return False
        # merge argv
        if attached_value is None:
            argv.extend(to_match_ref_argv)
        logger.debug('%s matched %s / %s', self, self.value, argv)
        return True

    def reset(self):
        value = self.value
        if value is not None:
            if value is True or value is False:
                self.value = False
            else:
                self.value = 0

        if self.ref is not None:
            self.ref.reset()

    # Note: this is kind of tricky:
    # make it different from Unit(Command, Unit())
    def dump_value(self):
        ref = self.ref
        return {'self': self.value,
                'ref': None if ref is None else ref.dump_value()}

    def load_value(self, value):
        self.value = value['self']
        if self.ref is not None:
            self.ref.load_value(value['ref'])

    def merge_value(self, values):
        opt_values = [each['self'] for each in values]
        ref_values = [each['ref'] for each in values]

        if len(opt_values) == 1:
            assert len(ref_values) == 1
            result = {'self': opt_values[0], 'ref': ref_values[0]}
        else:
            opt_value = sum(filter(None, opt_values))
            if all(x is None for x in ref_values):
                ref_value = None
            else:
                ref_value = self.ref.merge_value(ref_values)
            result = {'self': opt_value, 'ref': ref_value}
        return result

    @classmethod
    def convert_2_dict(cls, obj):
        ref = obj.ref
        result = super(Option, cls).convert_2_dict(obj)
        result['ref'] = (ref.convert_2_dict(ref) if ref is not None else None)
        return result

    @classmethod
    def convert_2_object(cls, dic):
        names = dic['names']
        cls_name = dic['__class__']
        assert cls_name == 'Option'
        default = dic['default']
        ref_value = dic['ref']
        ref = None if ref_value is None else Unit.convert_2_object(ref_value)
        # Not work on py2.6
        # return cls(*names, default=default, ref=ref)
        return cls(*names, **{'default': default, 'ref': ref})

    def __str__(self):
        result = super(Option, self).__str__()
        if self.ref:
            return '`%s %s`' % (result, self.ref)
        return '`%s`' % result

    def __repr__(self):
        return 'Option(%s, ref=%r)' % (', '.join(self.names), self.ref)

    def __eq__(self, other):
        if not isinstance(other, Option):
            return False

        equal = self.names.intersection(other.names)

        if equal:
            equal = (self.ref == other.ref)

        return equal


class Command(Atom):

    def __init__(self, *names, **kwargs):
        # assert all(self.type(x) == self.COMMAND for x in names)
        super(Command, self).__init__(*names, **kwargs)
        self.value = False

    def reset(self):
        value = self.value
        if isinstance(value, bool):
            self.value = False
        else:
            self.value = 0
        # self.value = False

    def match(self, argv, repeat_match):
        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False

        if not repeat_match and self.value:
            logger.debug('%s already has a value %s', self, self.value)
            return True

        current = argv.current()
        if current is None:
            logger.debug('argv ran out before matching %s', self)
            return False

        if argv.option_only:
            logger.debug('option only, %s skipped.', self)
            return False

        skip = 0
        if current == '--':
            argv.check_dash()
            if argv.auto_dashes and argv.dashes:
                current = argv.current(1)
                skip = 1
            else:
                logger.debug('%s matching %s failed', self, current)
                return False

        if current not in self.names or Atom.get_class(current) is Option:
            logger.debug('%s matching %s failed', self, current)
            return False

        if repeat_match:
            self.value += 1
        else:
            self.value = True

        # saver.save(self, argv)
        argv.next(skip)
        logger.debug('%s matched %s/%s', self, self.value, argv)
        return True

    def merge_value(self, values):
        if len(values) == 1:
            return values[0]
        else:
            return sum(filter(None, values))

    def get_value(self, appeared_only, in_repeat):
        if in_repeat:
            value = int(self.value)
        else:
            value = self.value

        # Not work on py2.6
        # return {name: value for name in self.names}
        result = {}
        for name in self.names:
            result[name] = value
        return result

    def get_sys_default_value(self, appeared_only, in_repeat):
        if in_repeat:
            value = 0
        else:
            value = False
        # Not work on py2.6
        # return {name: value for name in self.names}
        result = {}
        for name in self.names:
            result[name] = value
        return result


class Argument(Atom):

    def __init__(self, *names, **kwargs):
        super(Argument, self).__init__(*names, **kwargs)
        self.value = None

    def reset(self):
        if isinstance(self.value, list):
            del self.value[:]
        else:
            self.value = None

    def dump_value(self):
        value = self.value
        if isinstance(value, list):
            return list(value)
        return value

    def match(self, argv, repeat_match):
        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False

        if not repeat_match and (self.value is not None and self.value != []):
            logger.info('%s already has a value %s', self, self.value)
            return True

        if argv.option_only:
            logger.debug('option only, %s skipped.', self)
            return False

        current = argv.current()
        if current is None:
            logger.debug('argv ran out when matching %s', self)
            return False

        if current == '--':
            argv.check_dash()
            if argv.auto_dashes and argv.dashes:
                current = argv.current(1)
                # nothing left
                if current is None:
                    logger.debug('argv %s ran out when matching %s',
                                 argv, self)
                    return False
                # force to be command/arg
                if repeat_match:
                    if self.value is None:
                        self.value = [current]
                    else:
                        self.value.append(current)

                else:
                    self.value = current

                # saver.save(self, argv)
                argv.next(1)
                logger.debug('%s matched %s/%s', self, self.value, argv)
                return True
            else:
                logger.debug('%s matching %s failed', self, current)
                return False

        # check if it's `--flag=sth`

        if current.startswith('--') and '=' in current:
            opt, value = current.split('=', 1)
            if Atom.get_class(opt) is Option:
                logger.debug('%s matching %s failed', self, current)
                return False

        if Atom.get_class(current) is Option:
            logger.debug('%s matching %s failed', self, current)
            return False

        if repeat_match:
            if self.value is None:
                self.value = [current]
            else:
                self.value.append(current)

        else:
            self.value = current

        # saver.save(self, argv)
        argv.next()
        logger.debug('%s matched %s/%s', self, self.value, argv)
        return True

    def get_value(self, appeared_only, in_repeat):
        value = self.value
        if in_repeat:
            if value is None:
                value = []
            elif not isinstance(value, list):
                value = [value]
        # Not work on py2.6
        # return {name: value for name in self.names}
        result = {}
        for name in self.names:
            result[name] = value
        return result

    def get_sys_default_value(self, appeared_only, in_repeat):
        if in_repeat:
            value = []
        else:
            value = None
        # Not work on py2.6
        # return {name: value for name in self.names}
        result = {}
        for name in self.names:
            result[name] = value
        return result

    def merge_value(self, values):
        if len(values) == 1:
            return values[0]
        else:
            result = []
            for each in values:
                if each is None:
                    pass
                elif isinstance(each, list):
                    result.extend(each)
                else:
                    result.append(each)
            return result

    def __eq__(self, other):
        if isinstance(other, Argument):
            return True
        return False


class Unit(list):

    def __init__(self, *atoms, **kwargs):
        super(Unit, self).__init__(atoms)
        self.repeat = kwargs.get('repeat', False)

    def reset(self):
        for each in self:
            each.reset()

    def get_value(self, appeared_only, in_repeat):
        result = {}
        for each in self:
            this_result = each.get_value(appeared_only,
                                         in_repeat or self.repeat)
            repeated_keys = set(result).intersection(this_result)
            for key in repeated_keys:
                old_value = result[key]
                new_value = this_result[key]
                logger.debug('%s - old: %s, new: %s',
                             key, old_value, new_value)
                if (key.startswith('-') and
                        any(
                            isinstance(
                                x,
                                (int, NoneType))
                            for x in (old_value, new_value))):
                    # -1 - -1
                    if appeared_only and old_value == new_value == -1:
                        final_value = -1
                    elif set((old_value, new_value)).issubset((None, -1)):
                        final_value = []
                    # countable
                    else:
                        final_value = (old_value or 0) + (new_value or 0)
                elif all(isinstance(x, int) for x in (old_value, new_value)):
                    final_value = old_value + new_value
                else:
                    final_value = []
                    for each_value in (old_value, new_value):
                        if each_value is None:
                            pass
                        elif isinstance(each_value, list):
                            final_value.extend(each_value)
                        else:
                            final_value.append(each_value)
                logger.debug('merge %s value as %s', key, final_value)
                this_result[key] = final_value
            result.update(this_result)
        logger.debug('%s return value %s', self, result)
        return result

    def arg_range(self):
        if not self:
            return [0]

        all_range = (each.arg_range() for each in self)
        # all_range = list(all_range)
        producted = product(*all_range)
        # producted = list(producted)
        this_range = set(sum(x) for x in producted)

        if self.repeat:
            this_range.add(float('inf'))

        return list(this_range)

    def fix(self):
        if not self:
            return None
        if len(self) == 1:
            return self._fix_single_element()
        else:
            return self._fix_multi_element()

    def _fix_single_element(self):
        this_element = self[0].fix()
        # None, Atom, Unit, Either

        # None
        if this_element is None:
            return None

        # Atom, Unit, Either
        self[0] = this_element

        # Unit
        if isinstance(this_element, Unit):
            return self.fix_nest()

        # Atom
        return self

    def _fix_multi_element(self):
        result = []
        for each in self:
            fixed = each.fix()
            if fixed is not None:
                result.append(fixed)
        if not result:
            return None

        self[:] = result
        if len(result) == 1:
            return self._fix_single_element()
        else:
            return self

    def fix_nest(self):
        assert len(self) == 1, '%s need not fix nest' % self
        inside = self[0]
        if isinstance(inside, Unit):
            if len(inside) == 0:  # [()], ([]), (()), [[]]
                return None
            elif len(inside) == 1:  # [[sth]], ((sth)), ([sth]), [(sth)]
                innder_inside = inside[0].fix()
                if innder_inside is None:
                    return None
                if any(isinstance(x, Optional) for x in (self, inside)):
                    # [[sth]], ([sth]), [(sth)] -> (sth)
                    cls = Optional
                else:
                    # ((sth)) -> (sth)
                    cls = Required
                result = cls(innder_inside,
                             repeat=self.repeat or inside.repeat)
                return result.fix()

            repeat = self.repeat or inside.repeat
            # [(...)]
            if isinstance(self, Optional) and isinstance(inside, Required):
                fixed_inside = inside.fix()
                # [?]
                if fixed_inside is not inside:  # new
                    self[0] = fixed_inside
                    return self.fix()
                # inside does not need fix at all
                # [(...)] -> [(...)]
                return self
            # ([...]), ((...)), [[...]]
            else:
                inside.repeat = repeat
                return inside.fix()
        return self

    def push_option_ahead(self):
        options = []
        others = []
        for each in self:
            if isinstance(each, (Option, OptionsShortcut)):
                options.append(each)
            elif isinstance(each, Atom):
                others.append(each)
            elif each.push_option_ahead():
                options.append(each)
            else:
                others.append(each)
        self[:] = options + others
        return not others

    def _match_oneline(self, argv):
        # Though it's one line matching
        # It still need to deal with situation like:
        # `-b cmd1 -a cmd2 -b`
        # to match argv like `-b -a cmd1 cmd2 -b` or `-b cmd1 -a cmd2 -b`
        # it still check the line more than once

        old_status = None
        new_status = argv.status()
        matched_status = [isinstance(x, (Optional, OptionsShortcut))
                          for x in self]
        last_opt_or_arg = -1

        # the token is moving
        while argv.error is None and old_status != new_status and argv:
            old_status = new_status
            for index, each in enumerate(self):
                if not argv:
                    logger.debug('argv run out when matching %s', each)
                    break

                # saver.save(each, argv)
                argv.option_only = (index <= last_opt_or_arg)
                result = each.match(argv, False)
                if result:
                    old_matching_status = matched_status[index]
                    matched_status[index] = True
                    if not old_matching_status:
                        if isinstance(each, (Argument, Command)):
                            last_opt_or_arg = index
                            logger.debug('set last matched %s in %s at %s',
                                         each, self, index)
                        # jump to the first, avoid
                        # `Usage: cmd --flag <arg>`
                        # matching `cmd --flag sth` fails.
                        # elif isinstance(each, Option):
                        #     logger.debug('jump')
                        #     break
            new_status = argv.status()

        logger.debug('out of loop matching %s, argv %s', self, argv)
        return matched_status

    # TODO: balance the value for `<a>... <b>`
    def match_repeat(self, argv):
        # saver.save(self, argv)
        old_status = None
        new_status = argv.status()
        full_match_count = 0
        history_values = [self.dump_value()]
        logger.debug('matching %s repeatedly, start: %s', self, argv)
        while old_status != new_status and argv:
            self.reset()
            # saver.save(self, argv)
            old_status = new_status
            result = self.match_oneline(argv)
            if result:
                full_match_count += 1
            else:
                break
            history_values.append(self.dump_value())
            new_status = argv.status()

        if argv.error is not None:
            return 0

        logger.debug('matching %s %s time(s)', self, full_match_count)
        if full_match_count:
            if len(history_values) == 1:
                merged_value = history_values[0]
            else:
                merged_value = self.merge_value(history_values)
            logger.debug('restore merged value %s for %s', merged_value, self)
            self.load_value(merged_value)
        else:
            assert len(history_values) == 1
            self.load_value(history_values[0])
        return full_match_count

    # TODO: better solution
    def balance_value_for_ellipsis_args(self):
        # This method is very simple now. It only works for:
        # (<arg>)... <arg>
        # [<arg>]... <arg>
        # (<arg>)... <arg> <arg>
        # This even won't for:
        # (<arg>)... (<arg> <arg>)
        # (<arg> <arg>)... <arg>

        collected = []
        for index, element in enumerate(self):
            if (isinstance(element, Unit) and
                    element.repeat and
                    len(element) == 1 and
                    isinstance(element[0], Argument) and
                    element[0].value):

                rest = self[index + 1:]
                need_balance = []
                take_arg = True
                for each in rest:
                    if take_arg:
                        if isinstance(each, Argument):
                            need_balance.append(each)
                        else:
                            take_arg = False
                            # check if ret is matched
                            if isinstance(each, (Optional, OptionsShortcut)):
                                continue
                            elif each.match(Argv([], True, True,
                                                 True, True),
                                            False):
                                # use an empty token to test
                                continue
                            return False

                need_arg_number = len(need_balance)
                to_rent_arg = element[0]
                to_rent_value = to_rent_arg.value
                can_borrow_number = len(to_rent_value)
                if isinstance(element, Required):
                    can_borrow_number -= 1
                if can_borrow_number < need_arg_number:
                    return False

                logger.debug('balance %s(%s) -> %s', element, to_rent_arg,
                             need_balance)

                for need_value_arg in need_balance[::-1]:
                    need_value_arg.value = to_rent_value.pop()
                    logger.debug('set %s value %s',
                                 need_value_arg, need_value_arg.value)
                return True

            elif (not isinstance(element, (Optional, OptionsShortcut)) and
                    not element.match(Argv([], True, True, True, True),
                                      False)):
                return False

    # TODO: check if this is buggy
    def merge_value(self, value_list):
        result = []
        for index, values in enumerate(zip(*value_list)):
            ins = self[index]
            result.append(ins.merge_value(values))
        return result

    def dump_value(self):
        return [x.dump_value() for x in self]

    def load_value(self, value):
        for each, v in zip(self, value):
            each.load_value(v)

    def get_flat_list_value(self):
        result = []
        for each in self:
            if hasattr(each, 'get_flat_list_value'):
                result.extend(each.get_flat_list_value())
            else:
                assert isinstance(each, Argument)
                value = each.value
                # acturally argument has no default so far
                # default = each.default
                if value is None:
                    pass
                elif isinstance(value, list):
                    result.extend(value)
                else:
                    result.append(value)
        return result

    def get_sys_default_value(self, appeared_only, in_repeat):
        result = {}
        if in_repeat or self.repeat:
            for each in self:
                result.update(each.get_sys_default_value(appeared_only, True))
            return result

        for each in self:
            this_result = each.get_sys_default_value(appeared_only, False)
            common_keys = set(result).intersection(this_result)
            for key in common_keys:
                old_value = result[key]
                new_value = this_result[key]
                if all(isinstance(x, int)
                        for x in (old_value, new_value)):
                    this_result[key] = old_value + new_value
                else:
                    this_result[key] = []
            result.update(this_result)
        return result

    def expand_either(self):
        cls = Required if isinstance(self, Required) else Optional
        repeat = self.repeat
        if len(self) == 1 and isinstance(self[0], Either):
            return [cls(each, repeat=repeat).fix() for each in self[0]]

        result = []
        for expanded in product(*(x.expand_either() for x in self)):
            new = cls(*expanded, repeat=repeat).fix()
            result.append(new)

        return result

    @classmethod
    def convert_2_dict(cls, obj):
        return {
            '__class__': obj.__class__.__name__,
            'atoms': [x.convert_2_dict(x) for x in obj],
            'repeat': obj.repeat
        }

    @classmethod
    def convert_2_object(cls, dic):
        cls_name = dic['__class__']
        assert cls_name in ('Optional', 'Required')
        if cls_name == 'Optional':
            cls = Optional
        else:
            cls = Required
        atoms = (convert_2_object(x) for x in dic['atoms'])
        repeat = dic['repeat']
        # Not work on py2.6
        # return cls(*atoms, repeat=repeat)
        return cls(*atoms, **{'repeat': repeat})

    def __eq__(self, other):
        if self.repeat != other.repeat:
            return False

        if len(self) != len(other):
            return False

        return all(ts == tt for ts, tt in zip(self, other))

    def __repr__(self):
        return '%s(%s%s)' % (self.__class__.__name__,
                             ', '.join(map(repr, self)),
                             ', repeat=True' if self.repeat else '')


class Required(Unit):

    def match(self, argv, repeat_match):
        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False

        if not (repeat_match or self.repeat):
            logger.debug('try to match %s once, %s', self, argv)
            result = self.match_oneline(argv)
            logger.debug('%s matching status: %s', self, result)
            return (argv.error is None) and result

        logger.debug('try to match %s repeatedly', self)
        return self.match_repeat(argv) and argv.error is None

    def match_oneline(self, argv):
        # saver.save(self, argv)
        self_value = self.dump_value()
        argv_value = argv.dump_value()

        matched_status = self._match_oneline(argv)

        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False

        if all(matched_status):
            logger.debug('%s matched', self)
            return True
        logger.debug('%s matching failed %s / %s', self, matched_status, argv)
        if self.balance_value_for_ellipsis_args():
            logger.debug('%s balace value succeed', self)
            return True

        self.load_value(self_value)
        argv.load_value(argv_value)
        # saver.rollback(self, argv)
        return False

    def __eq__(self, other):
        if not isinstance(other, Required):
            return False

        return super(Required, self).__eq__(other)

    def __str__(self):
        return '(%s)%s' % (' '.join(map(str, self)),
                           '...' if self.repeat else '')


class Optional(Unit):

    def arg_range(self):
        this_range = super(Optional, self).arg_range()
        if 0 not in this_range:
            this_range.append(0)
        return this_range

    def match_oneline(self, argv):
        # saver.save(self, argv)
        self._match_oneline(argv)
        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False
        return True

    def match(self, argv, repeat_match):
        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False
        repeat = repeat_match or self.repeat
        logging.debug('matching %s with %s%s',
                      self, argv, ', repeatedly' if repeat else '')
        func = (self.match_repeat if repeat else self.match_oneline)

        func(argv)
        return argv.error is None

    def __eq__(self, other):
        if not isinstance(other, Optional):
            return False

        return super(Optional, self).__eq__(other)

    def __str__(self):
        if len(self) == 1:
            if isinstance(self[0], OptionsShortcut):
                if self.repeat:
                    return '%s...' % self[0]
                return str(self[0])

        return '[%s]%s' % \
               (' '.join(map(str, self)), '...' if self.repeat else '')


class OptionsShortcut(object):
    error = None

    def __init__(self, options):
        self._hide = set()
        self.options = options

    def set_hide(self, names):
        self._hide.update(names)

    def get_hide(self):
        return self._hide

    def need_hide(self, *name):
        return self._hide.intersection(name)

    def arg_range(self):
        logger.info('fixme: Unexpected call')
        return [0]

    def match(self, argv, repeat_match):
        options = self.options
        if argv.error is not None:
            logger.info('error in %s - %s', self, argv.error)
            return False

        hide = self._hide
        logger.debug('[options]/%s/%s try matching %s', options, hide, argv)
        for each in filter(
                lambda x: not hide.intersection(x[0].names), options):

            if not argv:
                logger.info('argv run out before matching [options] %s(-%s)',
                            options, self._hide)
                return argv.error is None
            logger.debug('[options] try %s matching %s', each, argv)
            each.match(argv, repeat_match)
            if argv.error is not None:
                return False
        return argv.error is None

    def get_value(self, appeared_only, in_repeat):
        options = self.options
        hide = self._hide
        result = {}  # developer should ensure no same options in [options]
        for each in filter(
                lambda x: not hide.intersection(x[0].names), options):
            result.update(each.get_value(appeared_only, in_repeat))
        return result

    def get_sys_default_value(self, appeared_only, in_repeat):
        options = self.options
        hide = self._hide
        result = {}
        for each in filter(
                lambda x: not hide.intersection(x[0].names), options):
            result.update(each.get_sys_default_value(appeared_only, in_repeat))
        return result

    def reset(self):
        for each in self.options:
            each.reset()

    @property
    def repeat(self):
        return False

    def fix(self):
        return self

    def dump_value(self):
        return [x.dump_value() for x in self.options]

    def load_value(self, value):
        for ins, val in zip(self.options, value):
            ins.load_value(val)

    def merge_value(self, values):
        options = self.options
        result = []
        for index, value in enumerate(zip(values)):
            ins = options[index]
            result.append(ins.merge_value(value))

        return result

    def expand_either(self):
        return [self]

    @classmethod
    def convert_2_dict(cls, obj):
        return {
            '__class__': obj.__class__.__name__,
            # 'ref': [x.convert_2_dict(x) for x in obj.ref],
            'hide': tuple(obj._hide),
        }

    @classmethod
    def convert_2_object(cls, dic, options):
        assert dic['__class__'] == 'OptionsShortcut'
        ins = OptionsShortcut(options)
        ins.set_hide(dic['hide'])
        return ins

    # TODO: better behavior
    def __eq__(self, other):
        if isinstance(other, OptionsShortcut):
            return True
        return False

    def __str__(self):
        return '`[options]`'

    def __repr__(self):
        return 'OptionsShortcut()'


# branch
class Either(list):
    error = None

    def __init__(self, *branch):
        assert(all(isinstance(x, Unit) for x in branch))
        super(Either, self).__init__(branch)
        self.matched_branch = -1

    def fix(self):
        if not self:
            return None
        result = []
        for each in self:
            fixed = each.fix()
            if fixed is not None:
                result.append(fixed)
        if not result:
            return None
        self[:] = result
        return self.fix_argument_only()

    def fix_argument_only(self):
        '''
        fix_argument_only() -> Either or Unit(Argument)
        `<arg> | ARG | <arg3>` ->
            `Required(Argument('<arg>', 'ARG', '<arg3>'))`
        `[<arg>] | [ARG] | [<arg3>]` ->
            `Optional(Argument('<arg>', 'ARG', '<arg3>'))`
        `(<arg>) | [ARG]` -> not change, return self
        `-a | --better` -> not change
        '''
        # for idx, branch in enumerate(self):
        #     if isinstance(branch[0], Either):
        #         self[idx] = branch.fix()
        first_type = type(self[0])
        if first_type not in (Required, Optional):
            return self
        for branch in self:
            if not (len(branch) == 1 and
                    isinstance(branch, first_type) and
                    isinstance(branch[0], Argument)):
                logger.debug('fix %r not change', self)
                return self
        else:
            first = self[0][0]
            for each in self:
                first.names.update(each[0].names)
            result = first_type(first)
            logger.debug('fix %r -> %r', self, result)
            return result

    def arg_range(self):
        result = set()
        for each in self:
            result.update(each.arg_range())
        return list(result)


def convert_2_dict(obj):
    return obj.convert_2_dict(obj)


def convert_2_object(dic, options={}):
    # never modify options
    cls_name = dic['__class__']
    if cls_name in ('Argument', 'Command', 'Option'):
        return Atom.convert_2_object(dic)
    elif cls_name in ('Optional', 'Required'):
        return Unit.convert_2_object(dic)
    elif cls_name == 'OptionsShortcut':
        return OptionsShortcut.convert_2_object(dic, options)
    else:
        raise ValueError('%s can not be converted to object', dic)



'''
This example uses docopt with the built in cmd module to demonstrate an
interactive command application.
Usage:
    my_program tcp <host> <port> [--timeout=<seconds>]
    my_program serial <port> [--baud=<n>] [--timeout=<seconds>]
    my_program (-i | --interactive)
    my_program (-h | --help | --version)

Options:
    -i, --interactive  Interactive Mode
    -h, --help         Show this screen and exit.
    --baud=<n>         Baudrate [default: 9600]
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__))



'''
Example of program which uses [options] shortcut in pattern.

Usage:
  options_shortcut_example.py [options] <port>

Options:
  -h --help                show this help message and exit
  --version                show version and exit
  -n, --number N           use N as a number
  -t, --timeout TIMEOUT    set timeout TIMEOUT seconds
  --apply                  apply changes to database
  -q                       operate in quiet mode
'''

from docpie import docpie

if __name__ == '__main__':
    print(docpie(__doc__, version='0.0.1'))



import sys
import logging

from docpie.error import DocpieExit, DocpieError
from docpie.parser import UsageParser, OptionParser
from docpie.element import convert_2_object, convert_2_dict
from docpie.tokens import Argv


try:
    StrType = basestring
except NameError:
    StrType = str

logger = logging.getLogger('docpie')


class Docpie(dict):

    # Docpie version
    # it's not a good idea but it can avoid loop importing
    _version = '0.2.2'

    option_name = 'Options:'
    usage_name = 'Usage:'
    doc = None

    case_sensitive = False
    auto2dashes = True
    name = None
    help = True
    version = None
    stdopt = True
    attachopt = True
    attachvalue = True
    options_first = False
    appeared_only = False
    extra = {}

    opt_names = []
    all_opt_names = set()
    require_arg_opt_names = set()

    def __init__(self, doc=None, help=True, version=None,
                 stdopt=True, attachopt=True, attachvalue=True,
                 auto2dashes=True, name=None, case_sensitive=False,
                 optionsfirst=False, appearedonly=False, extra={}):

        super(Docpie, self).__init__()

        # set config first
        self.set_config(
            stdopt=stdopt, attachopt=attachopt, attachvalue=attachvalue,
            auto2dashes=auto2dashes, name=name, case_sensitive=case_sensitive,
            optionsfirst=optionsfirst, appearedonly=appearedonly, extra={})

        if doc is not None:
            self.doc = doc
            self.help = help
            self.version = version
            self._init()

    def _init(self):
        usage_str, self.usage_text = UsageParser.parse(
            self.doc, self.usage_name, self.case_sensitive)
        opt_str, self.option_sections = OptionParser.parse(
            self.doc, self.option_name, self.case_sensitive)
        if self.usage_text is None:
            raise DocpieError('"Usage:" not found')

        opt_parser = OptionParser(
            opt_str, self.stdopt, self.attachopt, self.attachvalue)
        self.options = opt_parser.get_chain()
        opt_2_ins = opt_parser.name_2_instance

        self.usages, all_opts = UsageParser(
            usage_str, self.name, self.options, opt_2_ins,
            self.stdopt, self.attachopt, self.attachvalue
        ).get_chain_and_all_options()

        self.all_opt_names = set()
        self.require_arg_opt_names = set()

        for opt_ins in all_opts:
            self.all_opt_names.update(opt_ins.names)
            if opt_ins.ref:
                self.require_arg_opt_names.update(opt_ins.names)

        self.opt_names = [x[0].names for x in self.options]

        logger.debug(self.opt_names)

        self.set_config(help=self.help, version=self.version)

    def docpie(self, argv=None):
        """match the argv for each usages, return dict.

        if argv is None, it will use sys.argv instead.
        if argv is str, it will call argv.split() first.
        this function will check the options in self.extra and handle it first.
        Which means it may not try to match any usages because of the checking.
        """

        token = self._prepare_token(argv)
        # check first, raise after
        # so `-hwhatever` can trigger `-h` first
        self.check_flag_and_handler(token)

        if self.option_sections:
            help_msg = ('%s\n%s' %
                        (self.usage_text,
                         '\n'.join(self.option_sections.values())))
        else:
            help_msg = self.usage_text

        if token.error is not None:
            raise DocpieExit('%s\n\n%s' % (token.error, help_msg))

        matched, result, dashed = self._match(token)
        if not matched:
            if result is None:  # not matched
                msg = help_msg
            else:  # hit an error
                msg = '%s%s\n\n%s' % (
                    result,
                    ' Use "--help" to see more' if self.help else '',
                    self.usage_text)
            raise DocpieExit(msg)

        value = result.get_value(self.appeared_only, False)
        self.clear()
        self.update(value)
        if self.appeared_only:
            self._drop_non_appeared()

        logger.debug('get all matched value %s', self)
        rest = list(self.usages)  # a copy
        rest.remove(result)
        self._add_rest_value(rest)
        logger.debug('merged rest values, now %s', self)
        self._add_option_value()
        self._dashes_value(dashed)

        return dict(self)  # remove all other reference in this instance

    def _drop_non_appeared(self):
        for key, _ in filter(lambda k_v: k_v[1] == -1, dict(self).items()):
            self.pop(key)

    def _add_rest_value(self, rest):
        for each in rest:
            default_values = each.get_sys_default_value(
                self.appeared_only, False)
            logger.debug('get rest values %s -> %s', each, default_values)
            common_keys = set(self).intersection(default_values)

            for key in common_keys:
                default = default_values[key]
                valued = self[key]
                logger.debug('%s: default(%s), matched(%s)',
                             key, default, valued)

                if ((default is not True and default is not False) and
                        isinstance(default, int)):
                    valued = int(valued)
                elif isinstance(default, list):
                    if valued is None:
                        valued = []
                    elif isinstance(valued, list):
                        pass
                    else:
                        valued = [valued]

                logger.debug('set %s as %s', key, valued)
                default_values[key] = valued

            self.update(default_values)

    def _add_option_value(self):
        # add left option, add default value
        for each in self.options:
            option = each[0]
            names = option.names
            default = option.default
            this_value = option.value

            logger.debug('%s/%s/%s', option, default, this_value)

            name_in_value = names.intersection(self)
            if name_in_value:  # add default if necessary
                one_name = name_in_value.pop()
                value_in_usage = self[one_name]
                if not value_in_usage:  # need default
                    if default is None:  # no default, use old matched one
                        final_value = value_in_usage
                    elif (each.repeat or
                            (value_in_usage is not True and
                             value_in_usage is not False and
                             isinstance(value_in_usage, (int, list)))):
                        final_value = default.split()
                    else:
                        final_value = default
                else:
                    final_value = value_in_usage
                if option.ref is None and each.repeat:
                    final_value = int(final_value or 0)
            # just add this key-value. Note all option here never been matched
            elif self.appeared_only:
                continue
            else:
                ref = option.ref

                if default is not None:
                    if (each.repeat or
                            (this_value not in (True, False) and
                             isinstance(this_value, (int, list)))):
                        final_value = default.split()
                    else:
                        if ref is not None and max(ref.arg_range()) > 1:
                            final_value = default.split()
                        else:
                            final_value = default
                else:
                    if ref is not None:
                        arg_range = ref.arg_range()
                        # if min(arg_range) != 0:
                        #     # It requires at least a value
                        #     logger.info('%s expects value', option)
                        #     raise DocpieExit(DocpieException.usage_str)
                        if max(arg_range) == 1:
                            final_value = None
                        else:
                            assert max(arg_range) > 1
                            final_value = []
                    # ref is None
                    elif this_value is None:
                        final_value = 0 if each.repeat else False
                    else:
                        final_value = \
                            int(this_value) if each.repeat else this_value

            logger.debug('set %s value %s', names, final_value)
            # Not work on py2.6
            # value.update({name: final_value for name in names})
            final = {}
            for name in names:
                final[name] = final_value
            self.update(final)

    def _dashes_value(self, dashes):
        result = self['--'] if '--' in self else dashes
        if self.options_first:
            if result is True:
                result = False
            elif result is False:
                pass
            elif isinstance(result, int):
                result = max(0, result - 1)

        if self.auto2dashes:
            result = bool(result)

        self['--'] = result

    def _prepare_token(self, argv):
        if argv is None:
            argv = sys.argv
        elif isinstance(argv, StrType):
            argv = argv.split()

        # the things in extra may not be announced
        all_opt_names = set(self.all_opt_names)
        all_opt_names.update(self.extra.keys())
        token = Argv(argv[1:], self.auto2dashes or self.options_first,
                     self.stdopt, self.attachopt, self.attachvalue,
                     all_opt_names)
        token.formal(self.options_first)
        return token

    def _match(self, token):
        for each in self.usages:
            logger.debug('matching usage %s', each)
            argv_clone = token.clone()
            if each.match(argv_clone, False):
                logger.debug('matched usage %s, checking rest argv %s',
                             each, argv_clone)
                if (not argv_clone or
                        (argv_clone.auto_dashes and
                         list(argv_clone) == ['--'])):
                    argv_clone.check_dash()
                    logger.info('matched usage %s / %s', each, argv_clone)
                    return True, each, argv_clone.dashes

                each.reset()
                logger.info('matching %s left %s, checking failed',
                            each, argv_clone)
                continue

            elif argv_clone.error is None:
                each.reset()
                logger.info('failed matching usage %s / %s', each, argv_clone)

            else:
                logger.info('error in %s - %s', each, argv_clone.error)
                return False, argv_clone.error, argv_clone.dashes

        else:
            logger.info('none matched')
            return False, argv_clone.error, argv_clone.dashes

    def check_flag_and_handler(self, token):
        need_arg = self.require_arg_opt_names
        options = set()

        for ele in token:
            if self.auto2dashes and ele == '--':
                break
            if ele.startswith('-') and ele != '-':
                options.add(ele)

        for inputted in options:

            found = False
            for auto, handler in self.extra.items():

                if auto.startswith('--') and inputted.startswith('--'):
                    logger.debug('%s <~ %s', inputted, auto)
                    if '=' in inputted:
                        inputted = inputted.split('=', 1)[0]
                    if inputted == auto:
                        found = True
                        break

                elif auto[1] != '-' and inputted[1] != '-':
                    logger.debug('%s <~ %s', inputted, auto)
                    if self.stdopt:
                        attachopt = self.attachopt
                        break_upper = False
                        for index, attached_name in enumerate(inputted[1:]):
                            if not attachopt and index > 0:
                                break

                            logger.debug('check %s <~ %s', attached_name, auto)

                            stacked_name = '-' + attached_name
                            if stacked_name == auto:
                                found = True
                                logger.debug('find %s in %s', auto, inputted)

                            if stacked_name in need_arg:
                                break_upper = True
                                break

                        if found or break_upper:  # break upper loop
                            break
                    else:
                        found = (inputted == auto)

            if found:
                logger.info('find %s, auto handle it', auto)
                handler(self, auto)

    @staticmethod
    def help_handler(docpie, flag):
        """Default help(`--help`, `-h`) handler. print help string and exit.

        By default, flag startswith `--` will print the full `doc`,
        otherwith, print "Usage" section and "Option" section.
        """
        if flag.startswith('--'):
            print(docpie.doc)
        else:
            print(docpie.usage_text)
            option_sections = docpie.option_sections
            if option_sections:
                print('')
                print('\n'.join(option_sections.values()))
        sys.exit()

    @staticmethod
    def version_handler(docpie, flag):
        """Default `-v` and `--version` handler. print the verison and exit."""
        print(docpie.version)
        sys.exit()

    # Because it's divided from dict
    # json.dump(docpie, default=docpie.convert_2_dict) won't work
    # so convert to dict before JSONlizing
    def convert_2_dict(self):  # cls, self):
        """Convert Docpie into a JSONlizable dict.

        Use it in this way:
        pie = Docpie(__doc__)
        json.dumps(pie.convert_2_dict())

        Note the `extra` info will be lost if you costomize that,
        because a function is not JSONlizable.
        You can use `set_config(extra={...})` to set it back.
        """
        config = {
            'stdopt': self.stdopt,
            'attachopt': self.attachopt,
            'attachvalue': self.attachvalue,
            'auto2dashes': self.auto2dashes,
            'case_sensitive': self.case_sensitive,
            'appearedonly': self.appeared_only,
            'optionsfirst': self.options_first,
            'option_name': self.option_name,
            'usage_name': self.usage_name,
            'name': self.name,
            'help': self.help,
            'version': self.version
        }

        text = {
            'doc': self.doc,
            'usage_text': self.usage_text,
            'option_sections': self.option_sections,
        }

        option = [convert_2_dict(x) for x in self.options]

        usage = [convert_2_dict(x) for x in self.usages]

        return {
            '__version__': self._version,
            '__class__': 'Docpie',
            '__config__': config,
            '__text__': text,
            'option': option,
            'usage': usage,
            'option_names': [list(x) for x in self.opt_names],
            'all_option_names': list(self.all_opt_names),
            'requiring_argument_options': list(self.require_arg_opt_names)
        }

    @classmethod
    def convert_2_docpie(cls, dic):
        """Convert dict generated by `convert_2_dict` into Docpie instance

        You can do this:
        pie = Docpie(__doc__)
        clone_pie = json.loads(pie.convert_2_docpie(
            json.dumps(pie.convert_2_dict())
        ))

        Note if you changed `extra`, it will be lost.
        You can use `set_config(extra={...})` to set it back.
        """
        if '__version__' not in dic:
            raise ValueError('Not support old docpie data')

        data_version = int(dic['__version__'].replace('.', ''))
        this_version = int(cls._version.replace('.', ''))
        logger.debug('this: %s, old: %s', this_version, data_version)
        if data_version < this_version:
            raise ValueError('Not support old docpie data')
        assert dic['__class__'] == 'Docpie'
        config = dic['__config__']
        help = config.pop('help')
        version = config.pop('version')
        option_name = config.pop('option_name')
        usage_name = config.pop('usage_name')
        self = cls(None, **config)
        self.option_name = option_name
        self.usage_name = usage_name

        text = dic['__text__']
        self.doc = text['doc']
        self.usage_text = text['usage_text']
        self.option_sections = text['option_sections']

        self.opt_names = [set(x) for x in dic['option_names']]
        self.all_opt_names = set(dic['all_option_names'])
        self.require_arg_opt_names = set(dic['requiring_argument_options'])
        self.set_config(help=help, version=version)

        self.options = [convert_2_object(x) for x in dic['option']]

        self.usages = [convert_2_object(x, self.options) for x in dic['usage']]

        return self

    def set_config(self, **config):
        """Shadow all the current config."""
        reinit = False
        if 'stdopt' in config:
            stdopt = config.pop('stdopt')
            reinit = (stdopt != self.stdopt)
            self.stdopt = stdopt
        if 'attachopt' in config:
            attachopt = config.pop('attachopt')
            reinit = reinit or (attachopt != self.attachopt)
            self.attachopt = attachopt
        if 'attachvalue' in config:
            attachvalue = config.pop('attachvalue')
            reinit = reinit or (attachvalue != self.attachvalue)
            self.attachvalue = attachvalue
        if 'auto2dashes' in config:
            self.auto2dashes = config.pop('auto2dashes')
        if 'name' in config:
            name = config.pop('name')
            reinit = reinit or (name != self.name)
            self.name = name
        if 'help' in config:
            self.help = config.pop('help')
            self._set_or_remove_extra_handler(
                self.help, ('--help', '-h'), self.help_handler)
        if 'version' in config:
            logger.debug(config['version'])
            self.version = config.pop('version')
            self._set_or_remove_extra_handler(
                self.version is not None,
                ('--version', '-v'),
                self.version_handler)
        if 'case_sensitive' in config:
            case_sensitive = config.pop('case_sensitive')
            reinit = reinit or (case_sensitive != self.case_sensitive)
            self.case_sensitive = case_sensitive
        if 'optionsfirst' in config:
            self.options_first = config.pop('optionsfirst')
        if 'appearedonly' in config:
            self.appeared_only = config.pop('appearedonly')
        if 'extra' in config:
            self.extra.update(config.pop('extra'))

        if config:  # should be empty
            raise ValueError(
                '%s is/are not accepted key argument(s)' % list(config.keys()))

        if self.doc is not None and reinit:
            logger.info('changed config require reinit instance')
            self._init()

    def _set_or_remove_extra_handler(self, set_handler, find_order, handler):
        for flag in find_order:
            alias = self.find_flag_alias(flag)
            if alias is not None:
                alias.add(flag)
                for each in alias:
                    if set_handler:
                        logger.info('set %s hanlder', each)
                        self.extra[each] = handler
                    else:
                        logger.info('remove %s hanlder', each)
                        self.extra.pop(each, None)
                break
        else:
            for flag in find_order:
                if set_handler:
                    logger.info('set %s hanlder', flag)
                    self.extra[flag] = handler
                else:
                    logger.info('remove %s hanlder', flag)
                    self.extra.pop(flag, None)

    def find_flag_alias(self, flag):
        """Return alias set of a flag; return None if flag is not defined in
        "Options".
        """
        for each in self.opt_names:
            if flag in each:
                result = set(each)  # a copy
                result.remove(flag)
                return result
        return None

    def set_auto_handler(self, flag, handler):
        """Set pre-auto-handler for a flag.

        the handler must accept two argument: first the `pie` which
        referent to the current `Docpie` instance, second, the `flag`
        which is the flag found in `argv`.

        Different from `extra` argument, this will set the alias
        option you defined in `Option` section with the same
        behavior.
        """
        assert flag.startswith('-') and flag not in ('-', '--')
        alias = self.find_flag_alias(flag) or []
        self.extra[flag] = handler
        for each in alias:
            self.extra[each] = handler

    def preview(self, stream=sys.stdout):
        """A quick preview of docpie. Print all the parsed object"""

        write = stream.write

        write(('[Quick preview of Docpie %s]' % self._version).center(80, '='))
        write('\n')

        write(' str '.center(80, '-'))
        write('\nUsage:\n')
        for each in self.usages:
            write('%s\n' % each)
        write('\nOptions:\n')
        for each in self.options:
            write('%s\n' % each)

        write(' repr '.center(80, '-'))
        write('\nUsage:\n')
        for each in self.usages:
            write('%r\n' % each)
        write('\n\nOptions:\n')
        for each in self.options:
            write('%r\n' % each)

        write(' auto handlers '.center(80, '-'))
        write('\n')
        for key, value in self.extra.items():
            write('%s %s\n' % (key, value))

    def __str__(self):
        return '{%s}' % ',\n '.join('%r: %r' % i for i in sorted(self.items()))
